import { __rest, __awaiter, __generator, __extends, __spreadArray, __read } from 'tslib';
import { propertyMetadataCache, isBrowser, AbstractRendererPlugin, AbstractRenderer } from '@antv/g-lite';
import * as DomInteraction from '@antv/g-plugin-dom-interaction';
export { DomInteraction };
import * as SVGPicker from '@antv/g-plugin-svg-picker';
export { SVGPicker };
import * as SVGRenderer from '@antv/g-plugin-svg-renderer';
import { createSVGElement } from '@antv/g-plugin-svg-renderer';
export { SVGRenderer };
import { isString } from '@antv/util';

var SVGContextService = /** @class */ (function () {
    function SVGContextService(context) {
        this.context = context;
        this.canvasConfig = context.config;
    }
    SVGContextService.prototype.init = function () {
        var _a = this.canvasConfig, container = _a.container, doc = _a.document, devicePixelRatio = _a.devicePixelRatio;
        // create container
        this.$container = isString(container)
            ? (doc || document).getElementById(container)
            : container;
        if (this.$container) {
            if (!this.$container.style.position) {
                this.$container.style.position = 'relative';
            }
            var $namespace = createSVGElement('svg', doc);
            $namespace.setAttribute('width', "".concat(this.canvasConfig.width));
            $namespace.setAttribute('height', "".concat(this.canvasConfig.height));
            this.$container.appendChild($namespace);
            this.$namespace = $namespace;
        }
        // use user-defined dpr first
        var dpr = devicePixelRatio || (isBrowser && window.devicePixelRatio) || 1;
        dpr = dpr >= 1 ? Math.ceil(dpr) : 1;
        this.dpr = dpr;
    };
    // @ts-ignore
    SVGContextService.prototype.getDomElement = function () {
        return this.$namespace;
    };
    SVGContextService.prototype.getContext = function () {
        return this.$namespace;
    };
    SVGContextService.prototype.getDPR = function () {
        return this.dpr;
    };
    SVGContextService.prototype.getBoundingClientRect = function () {
        var _a;
        return (_a = this.$namespace) === null || _a === void 0 ? void 0 : _a.getBoundingClientRect();
    };
    SVGContextService.prototype.destroy = function () {
        // destroy context
        if (this.$container && this.$namespace && this.$namespace.parentNode) {
            this.$container.removeChild(this.$namespace);
        }
    };
    SVGContextService.prototype.resize = function (width, height) {
        // SVG should ignore DPR
        if (this.$namespace) {
            this.$namespace.setAttribute('width', "".concat(width));
            this.$namespace.setAttribute('height', "".concat(height));
        }
    };
    SVGContextService.prototype.applyCursorStyle = function (cursor) {
        if (this.$container) {
            this.$container.style.cursor = cursor;
        }
    };
    SVGContextService.prototype.generateCSSText = function (animationName, selector, keyframes, timing, prefixes) {
        if (prefixes === void 0) { prefixes = {}; }
        var duration = timing.duration, easing = timing.easing, delay = timing.delay, direction = timing.direction, iterations = timing.iterations, fill = timing.fill;
        return ("@keyframes ".concat(animationName, "{").concat(keyframes
            .map(function (keyframe) {
            keyframe.offset; keyframe.composite; var computedOffset = keyframe.computedOffset; keyframe.easing; var rest = __rest(keyframe, ["offset", "composite", "computedOffset", "easing"]);
            var rules = Object.keys(rest)
                .map(function (attributeName) {
                return "".concat(attributeName, ":").concat(prefixes[attributeName] || '').concat(rest[attributeName], ";");
            })
                .join('');
            return "".concat(computedOffset * 100, "%{").concat(rules, "}");
        })
            .join(''), "}") +
            "".concat(selector, " {animation: ").concat(animationName, " ").concat(duration, "ms ").concat(easing, " ").concat(delay, "ms ").concat(iterations === Infinity ? 'infinite' : iterations, " ").concat(direction, " ").concat(fill, ";}"));
    };
    SVGContextService.prototype.toDataURL = function (options) {
        return __awaiter(this, void 0, void 0, function () {
            var cloneNode, doc, animationCounter, $style, svgDocType, svgDoc;
            var _this = this;
            return __generator(this, function (_a) {
                cloneNode = this.$namespace.cloneNode(true);
                doc = this.canvasConfig.document;
                animationCounter = 0;
                $style = null;
                this.context.renderingContext.root.forEach(function (object) {
                    var animations = object.getAnimations();
                    if (animations.length) {
                        if (!$style) {
                            // export animations to <style>, using CSS Transformation
                            $style = (doc || document).createElement('style');
                            cloneNode.appendChild($style);
                        }
                        // @ts-ignore
                        var svgElement_1 = object.elementSVG;
                        var selfSelector_1 = "#".concat(svgElement_1.$el.id);
                        var groupSelector_1 = "#".concat(svgElement_1.$groupEl.id);
                        var selfCssText_1 = '';
                        var groupCssText_1 = '';
                        animations.forEach(function (animation) {
                            var keyframes = animation.effect.getKeyframes();
                            // split attributes into self and group
                            if (keyframes.length) {
                                var selfAttributes_1 = [];
                                var groupAttributes_1 = [];
                                var _a = keyframes[0]; _a.offset; _a.composite; _a.computedOffset; _a.easing; _a.transformOrigin; var rest = __rest(_a, ["offset", "composite", "computedOffset", "easing", "transformOrigin"]);
                                Object.keys(rest).forEach(function (attributeName) {
                                    var _a;
                                    if (attributeName === 'transform') {
                                        groupAttributes_1.push(attributeName);
                                    }
                                    var inherited = !!((_a = propertyMetadataCache[attributeName]) === null || _a === void 0 ? void 0 : _a.inh);
                                    if (inherited) {
                                        groupAttributes_1.push(attributeName);
                                    }
                                    else if (attributeName !== 'transform') {
                                        selfAttributes_1.push(attributeName);
                                    }
                                });
                                if (groupAttributes_1.length) {
                                    var keyframesWithGroup = keyframes.map(function (keyframe) {
                                        var offset = keyframe.offset, composite = keyframe.composite, computedOffset = keyframe.computedOffset, easing = keyframe.easing; keyframe.transformOrigin; var rest = __rest(keyframe, ["offset", "composite", "computedOffset", "easing", "transformOrigin"]);
                                        var ret = { offset: offset, composite: composite, computedOffset: computedOffset, easing: easing };
                                        Object.keys(rest).forEach(function (attributeName) {
                                            if (groupAttributes_1.includes(attributeName)) {
                                                ret[attributeName] = keyframe[attributeName];
                                            }
                                        });
                                        return ret;
                                    });
                                    groupCssText_1 += _this.generateCSSText("a".concat(animationCounter++), groupSelector_1, keyframesWithGroup, animation.effect.getComputedTiming(), { transform: svgElement_1.$groupEl.getAttribute('transform') });
                                }
                                if (selfAttributes_1.length) {
                                    var keyframesWithSelf = keyframes.map(function (keyframe) {
                                        var offset = keyframe.offset, composite = keyframe.composite, computedOffset = keyframe.computedOffset, easing = keyframe.easing; keyframe.transformOrigin; var rest = __rest(keyframe, ["offset", "composite", "computedOffset", "easing", "transformOrigin"]);
                                        var ret = { offset: offset, composite: composite, computedOffset: computedOffset, easing: easing };
                                        Object.keys(rest).forEach(function (attributeName) {
                                            if (selfAttributes_1.includes(attributeName)) {
                                                ret[attributeName] = keyframe[attributeName];
                                            }
                                        });
                                        return ret;
                                    });
                                    selfCssText_1 += _this.generateCSSText("a".concat(animationCounter++), selfSelector_1, keyframesWithSelf, animation.effect.getComputedTiming());
                                }
                            }
                        });
                        $style.textContent += selfCssText_1 + groupCssText_1;
                    }
                });
                svgDocType = document.implementation.createDocumentType('svg', '-//W3C//DTD SVG 1.1//EN', 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd');
                svgDoc = document.implementation.createDocument('http://www.w3.org/2000/svg', 'svg', svgDocType);
                svgDoc.replaceChild(cloneNode, svgDoc.documentElement);
                return [2 /*return*/, "data:image/svg+xml;charset=utf8,".concat(encodeURIComponent(new XMLSerializer().serializeToString(svgDoc)))];
            });
        });
    };
    return SVGContextService;
}());

var ContextRegisterPlugin = /** @class */ (function (_super) {
    __extends(ContextRegisterPlugin, _super);
    function ContextRegisterPlugin() {
        var _this = _super.apply(this, __spreadArray([], __read(arguments), false)) || this;
        _this.name = 'svg-context-register';
        return _this;
    }
    ContextRegisterPlugin.prototype.init = function () {
        // @ts-ignore
        this.context.ContextService = SVGContextService;
    };
    ContextRegisterPlugin.prototype.destroy = function () {
        delete this.context.ContextService;
    };
    return ContextRegisterPlugin;
}(AbstractRendererPlugin));

var Renderer = /** @class */ (function (_super) {
    __extends(Renderer, _super);
    function Renderer(config) {
        if (config === void 0) { config = {}; }
        var _this = _super.call(this, config) || this;
        _this.registerPlugin(new ContextRegisterPlugin());
        _this.registerPlugin(new SVGRenderer.Plugin({
            outputSVGElementId: config.outputSVGElementId,
        }));
        _this.registerPlugin(new DomInteraction.Plugin());
        _this.registerPlugin(new SVGPicker.Plugin());
        return _this;
    }
    return Renderer;
}(AbstractRenderer));

export { Renderer };
//# sourceMappingURL=index.esm.js.map
